@@ -40,10 +40,12 @@ class PerExampleAccuracy(tf.keras.metrics.Accuracy):
   def __init__(self, name: str = 'per_example_accuracy', **kwargs):
     super(PerExampleAccuracy, self).__init__(name=name, **kwargs)
 
-  def update_state(self,
-                   y_true: tf.Tensor,
-                   y_pred_scores: tf.Tensor,
-                   sample_weight: Optional[tf.Tensor] = None) -> None:
+  def update_state(
+      self,
+      y_true: tf.Tensor,
+      y_pred_scores: tf.Tensor,
+      sample_weight: Optional[tf.Tensor] = None,
+  ) -> None:
     """Accumulates running per-example accuracy."""
     del sample_weight  # We use the mask calculated here instead.
 
@@ -51,7 +53,8 @@ class PerExampleAccuracy(tf.keras.metrics.Accuracy):
     y_true = tf.cast(left_shift_sequence(y_true), dc_constants.TF_DATA_TYPE)
     # Convert pred scores and left shift.
     y_pred = tf.cast(
-        tf.argmax(y_pred_scores, axis=-1), dc_constants.TF_DATA_TYPE)
+        tf.argmax(y_pred_scores, axis=-1), dc_constants.TF_DATA_TYPE
+    )
     y_pred = left_shift_sequence(y_pred)
 
     # Count matching positions per row.
@@ -66,22 +69,23 @@ class PerClassAccuracy(tf.keras.metrics.Accuracy):
   """Compute per-position accuracy for the given class."""
 
   def __init__(self, class_value: int, name: Optional[str] = None, **kwargs):
-    if not name:
-      name = str(class_value)
-    name = f'per_class_accuracy_{name}'
     self.class_value = class_value
     super(PerClassAccuracy, self).__init__(name=name, **kwargs)
 
-  def update_state(self,
-                   y_true: tf.Tensor,
-                   y_pred_scores: tf.Tensor,
-                   sample_weight: Optional[tf.Tensor] = None) -> None:
+  def update_state(
+      self,
+      y_true: tf.Tensor,
+      y_pred_scores: tf.Tensor,
+      sample_weight: Optional[tf.Tensor] = None,
+  ) -> None:
     """Accumulates running per-position accuracy for the given class."""
     del sample_weight  # We use the mask calculated here instead.
     y_pred = tf.cast(
-        tf.argmax(y_pred_scores, axis=-1), dc_constants.TF_DATA_TYPE)
+        tf.argmax(y_pred_scores, axis=-1), dc_constants.TF_DATA_TYPE
+    )
     mask = tf.cast(
-        tf.equal(y_true, self.class_value), dc_constants.TF_DATA_TYPE)
+        tf.equal(y_true, self.class_value), dc_constants.TF_DATA_TYPE
+    )
     super().update_state(y_true, y_pred, sample_weight=mask)
 
 
@@ -94,9 +98,8 @@ def left_shift_sequence(y_true: tf.Tensor) -> tf.int32:
 
   Returns:
     left shifted y_true
-
   """
-  gap_token = dc_constants.VOCAB.find(dc_constants.GAP_OR_PAD)
+  gap_token = dc_constants.SEQ_VOCAB.find(dc_constants.GAP)
   shape = tf.shape(y_true)
   seq_length = shape[1]
 
@@ -105,8 +108,9 @@ def left_shift_sequence(y_true: tf.Tensor) -> tf.int32:
   # an index by the seq length, perform sort, then subtract to return
   # original index.
   sort_order = tf.sort(tf.where(y_true != gap_token, ixs, seq_length + ixs))
-  sort_order = tf.where(sort_order < seq_length, sort_order,
-                        sort_order - seq_length)
+  sort_order = tf.where(
+      sort_order < seq_length, sort_order, sort_order - seq_length
+  )
   y_true_left_aligned = tf.gather(y_true, sort_order, axis=1, batch_dims=-1)
   return y_true_left_aligned
 
@@ -116,9 +120,9 @@ SubsCostFn = Callable[[tf.Tensor, tf.Tensor], tf.Tensor]
 InsCostFn = Callable[[tf.Tensor], tf.Tensor]
 
 
-def xentropy_subs_cost_fn(y_true: tf.Tensor,
-                          y_pred: tf.Tensor,
-                          eps: float = 1e-7) -> tf.Tensor:
+def xentropy_subs_cost_fn(
+    y_true: tf.Tensor, y_pred: tf.Tensor, eps: float = 1e-7
+) -> tf.Tensor:
   """Pointwise cross-entropy substitution cost function for alignment loss.
 
   Args:
@@ -196,9 +200,9 @@ def xentropy_ins_cost_fn(y_pred: tf.Tensor, eps=1e-7) -> tf.Tensor:
 
   Returns:
     A tf.Tensor<float>[batch, n] such that out[b][l] represents the
-    cross-entropy loss between dc_constants.GAP_OR_PAD and y_pred[b][l].
+    cross-entropy loss between dc_constants.PAD and y_pred[b][l].
   """
-  gap_token = dc_constants.VOCAB.find(dc_constants.GAP_OR_PAD)
+  gap_token = dc_constants.SEQ_VOCAB.find(dc_constants.GAP)
   ins_scores = tf.clip_by_value(y_pred[..., gap_token], eps, 1 - eps)
   return -tf.math.log(ins_scores)
 
@@ -290,7 +294,8 @@ class AlignmentLoss(tf.keras.losses.Loss):
       del_cost: float = 1.0,
       loss_reg: Optional[float] = 1.0,
       width: Optional[int] = None,
-      reduction: tf.keras.losses.Reduction = tf.keras.losses.Reduction.AUTO):
+      reduction: tf.keras.losses.Reduction = tf.keras.losses.Reduction.AUTO,
+  ):
     super(AlignmentLoss, self).__init__(reduction=reduction)
     self.subs_cost_fn = subs_cost_fn
     self.ins_cost_fn = ins_cost_fn
@@ -314,22 +319,21 @@ class AlignmentLoss(tf.keras.losses.Loss):
       A tuple (y_true_oh, seq_lens) such that
         +  y_true_oh is a tf.Tensor<dtype>[batch, m, n_tokens], where n_tokens
            is the number of tokens in dc_constants.VOCAB. It contains a one-hot
-           representation of the input y_true, with dc_constants.GAP_OR_PAD
-           tokens removed and extra dc_constants.GAP_OR_PAD tokens appended if
+           representation of the input y_true, with dc_constants.PAD
+           tokens removed and extra dc_constants.PAD tokens appended if
            necessary.
         +  seq_lens is a tf.Tensor<int>[batch] containing the length of each
            label sequence in y_true, excluding any pad and gap tokens.
     """
     # Ensures y_true is of integer type.
     y_true = tf.cast(y_true, tf.int32)
-    # Removes internal gaps, shifting sequences left and adding padding when
-    # necessary.
+    # Removes internal gaps, shifting sequences left.
     y_true = left_shift_sequence(y_true)
-    # Computes per-example label sequence length, excluding padding.
-    pad_token = dc_constants.VOCAB.find(dc_constants.GAP_OR_PAD)
+    # Computes per-example label sequence length.
+    pad_token = dc_constants.SEQ_VOCAB.find(dc_constants.GAP)
     seq_lens = tf.reduce_sum(tf.cast(y_true != pad_token, y_true.dtype), -1)
     # Converts y_true to one-hot.
-    n_tokens = len(dc_constants.VOCAB)
+    n_tokens = len(dc_constants.SEQ_VOCAB)
     y_true_oh = tf.one_hot(y_true, depth=n_tokens, dtype=dtype)
     return y_true_oh, seq_lens
 
@@ -372,11 +376,14 @@ class AlignmentLoss(tf.keras.losses.Loss):
     # Initializes recursion.
     v_opt = tf.fill([b], inf)
     v_p2 = tf.pad(tf.fill([m - 1, b], inf), [[1, 0], [0, 0]])
-    v_p1 = tf.concat([
-        tf.slice(ins_costs[0], [0, 0], [1, b]),
-        tf.fill([1, b], del_cost),
-        tf.fill([m - 1, b], inf)
-    ], 0)
+    v_p1 = tf.concat(
+        [
+            tf.slice(ins_costs[0], [0, 0], [1, b]),
+            tf.fill([1, b], del_cost),
+            tf.fill([m - 1, b], inf),
+        ],
+        0,
+    )
     # Precomputes auxiliary (constant) tensors used during the recursion.
     i_range = tf.range(m + 1, dtype=tf.int32)
     k_end = seq_lens + n  # Indexes antidiagonal containing last entry, w/o pad.
@@ -395,8 +402,9 @@ class AlignmentLoss(tf.keras.losses.Loss):
       o_d = v_p2 + del_cost  # [m, b]
 
       v_p1 = tf.concat(
-          [tf.slice(o_i, [0, 0], [1, b]),
-           minop(tf.stack([o_m, o_i[1:], o_d]))], 0)
+          [tf.slice(o_i, [0, 0], [1, b]), minop(tf.stack([o_m, o_i[1:], o_d]))],
+          0,
+      )
       v_p1 = tf.where(inv_mask, v_p1, inf)
       v_opt = tf.where(k_end == k, tf.gather_nd(v_p1, nd_indices), v_opt)
 
@@ -431,17 +439,20 @@ class AlignmentLoss(tf.keras.losses.Loss):
     width = tf.cast(self.width, dtype=tf.int32)
     n_diag = 2 * width + 1
     diags = tf.linalg.diag_part(
-        input_v, k=(-width, width), padding_value=inf, align='LEFT_LEFT')
+        input_v, k=(-width, width), padding_value=inf, align='LEFT_LEFT'
+    )
     weave = tf.reshape(
         tf.stack([diags, tf.fill(tf.shape(diags), inf)], -1),
-        [batch, n_diag, -1])
+        [batch, n_diag, -1],
+    )
     woven_band_tr = inf * tf.ones((n_diag, batch, 2 * len_v))
     for diff in tf.range(-width, width + 1):
       i = diff + width
       abs_diff = tf.abs(diff)
       padded_weave = tf.roll(weave[:, n_diag - 1 - i], abs_diff, -1)
-      woven_band_tr = tf.tensor_scatter_nd_update(woven_band_tr, [[i]],
-                                                  padded_weave[tf.newaxis, ...])
+      woven_band_tr = tf.tensor_scatter_nd_update(
+          woven_band_tr, [[i]], padded_weave[tf.newaxis, ...]
+      )
     return tf.transpose(woven_band_tr, (1, 2, 0))[:, :-1, :]
 
   def index_ending_band(self, len_1, seq_lens):
@@ -452,14 +463,18 @@ class AlignmentLoss(tf.keras.losses.Loss):
     sum_index = i + j
     diff_index = j - i + self.width
     range_batch = tf.range(batch)
-    return tf.concat([
-        range_batch[..., tf.newaxis], sum_index[..., tf.newaxis],
-        diff_index[..., tf.newaxis]
-    ],
-                     axis=-1)
+    return tf.concat(
+        [
+            range_batch[..., tf.newaxis],
+            sum_index[..., tf.newaxis],
+            diff_index[..., tf.newaxis],
+        ],
+        axis=-1,
+    )
 
-  def banded_alignment(self, subs_costs, ins_costs, del_cost, seq_lens, inf,
-                       dtype):
+  def banded_alignment(
+      self, subs_costs, ins_costs, del_cost, seq_lens, inf, dtype
+  ):
     """Computes the alignment score values, with a band-restriction on the path.
 
     Args:
@@ -480,22 +495,26 @@ class AlignmentLoss(tf.keras.losses.Loss):
     len_2 = tf.shape(subs_costs)[2]
     val_trans = tf.zeros((len_1 + 1, len_2 + 1, batch))
     updates = [
-        del_cost * tf.tile(
+        del_cost
+        * tf.tile(
             tf.range(len_1 + 1, dtype=tf.float32)[..., tf.newaxis],
-            multiples=[1, batch])
+            multiples=[1, batch],
+        )
     ]
     val_trans = tf.tensor_scatter_nd_update(val_trans, [[0]], updates)
     for i in tf.range(1, len_1 + 1):
       previous_row = val_trans[i - 1, 0]
       val_trans = tf.tensor_scatter_nd_update(
-          val_trans, [[i, 0]], [previous_row + ins_costs[:, i - 1]])
+          val_trans, [[i, 0]], [previous_row + ins_costs[:, i - 1]]
+      )
       values = tf.transpose(val_trans, [2, 1, 0])
     input_band = self.weave_band(values, inf)
     subs_band = self.weave_band(subs_costs, inf)
-    ins_costs_pad = tf.pad(ins_costs, [[0, 0], [1, 0]], constant_values=0.)
+    ins_costs_pad = tf.pad(ins_costs, [[0, 0], [1, 0]], constant_values=0.0)
     # TODO: uphere
     insert_expand = tf.tile(
-        ins_costs_pad[:, tf.newaxis, :], multiples=[1, len_1 + 1, 1])
+        ins_costs_pad[:, tf.newaxis, :], multiples=[1, len_1 + 1, 1]
+    )
     insert_band = self.weave_band(insert_expand, inf)
     length = tf.shape(input_band)[1]
     # Sets up reduction operators.
@@ -506,18 +525,23 @@ class AlignmentLoss(tf.keras.losses.Loss):
       minop = lambda t: -loss_reg * tf.reduce_logsumexp(-t / loss_reg, axis=-1)
     for k in tf.range(2, length):
       input_minus_one = tf.pad(
-          input_band[..., :-1], [[0, 0], [0, 0], [1, 0]], constant_values=inf)
+          input_band[..., :-1], [[0, 0], [0, 0], [1, 0]], constant_values=inf
+      )
       input_plus_one = tf.pad(
-          input_band[..., 1:], [[0, 0], [0, 0], [0, 1]], constant_values=inf)
-      min_tens = tf.stack([
-          input_band[:, k - 2, :] + subs_band[:, k - 2, :],
-          input_plus_one[:, k - 1, :] + del_cost,
-          input_minus_one[:, k - 1, :] + insert_band[:, k, :],
-      ],
-                          axis=-1)
+          input_band[..., 1:], [[0, 0], [0, 0], [0, 1]], constant_values=inf
+      )
+      min_tens = tf.stack(
+          [
+              input_band[:, k - 2, :] + subs_band[:, k - 2, :],
+              input_plus_one[:, k - 1, :] + del_cost,
+              input_minus_one[:, k - 1, :] + insert_band[:, k, :],
+          ],
+          axis=-1,
+      )
       insert_mins = minop(min_tens)
       input_trans = tf.tensor_scatter_nd_update(
-          tf.transpose(input_band, [1, 0, 2]), [[k]], [insert_mins])
+          tf.transpose(input_band, [1, 0, 2]), [[k]], [insert_mins]
+      )
       input_band = tf.transpose(input_trans, [1, 0, 2])
     fetch_indices = self.index_ending_band(len_1, seq_lens)
     return tf.gather_nd(input_band, fetch_indices)
@@ -595,8 +619,8 @@ def preprocess_y_true_metric(y_true: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
   Returns:
     A tuple (y_true, y_true_lens) such that
       +  y_true is a tf.Tensor<int>[batch, m]. It contains the input y_true,
-          with dc_constants.GAP_OR_PAD tokens removed and extra
-          dc_constants.GAP_OR_PAD tokens appended if necessary.
+          with dc_constants.GAP tokens removed and extra
+          dc_constants.GAP tokens appended if necessary.
       +  y_true_lens is a tf.Tensor<int>[batch] containing the length of each
           label sequence in y_true, excluding any pad and gap tokens.
   """
@@ -606,7 +630,7 @@ def preprocess_y_true_metric(y_true: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
   # necessary.
   y_true = left_shift_sequence(y_true)
   # Computes per-example label sequence length, excluding padding.
-  pad_token = dc_constants.VOCAB.find(dc_constants.GAP_OR_PAD)
+  pad_token = dc_constants.SEQ_VOCAB.find(dc_constants.GAP)
   y_true_lens = tf.reduce_sum(tf.cast(y_true != pad_token, y_true.dtype), -1)
   return y_true, y_true_lens
 
@@ -622,8 +646,8 @@ def preprocess_y_pred_metric(y_pred: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
     A tuple (y_pred, y_pred_lens) such that
       +  y_pred is a tf.Tensor<int>[batch, m]. It contains the most likely
           token at each position of the input y_pred, with
-          dc_constants.GAP_OR_PAD tokens removed and extra
-          dc_constants.GAP_OR_PAD tokens appended if necessary.
+          dc_constants.PAD tokens removed and extra
+          dc_constants.PAD tokens appended if necessary.
       +  y_pred_lens is a tf.Tensor<int>[batch] containing the length of each
           predicted sequence in y_pred, excluding any pad and gap tokens.
   """
@@ -631,11 +655,10 @@ def preprocess_y_pred_metric(y_pred: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
   y_pred = tf.argmax(y_pred, axis=-1)
   # Ensures y_pred is of integer type.
   y_pred = tf.cast(y_pred, tf.int32)
-  # Removes internal gaps, shifting sequences left and adding padding when
-  # necessary.
+  # Removes internal gaps, shifting sequences left when necessary.
   y_pred = left_shift_sequence(y_pred)
   # Computes per-example label sequence length, excluding padding.
-  pad_token = dc_constants.VOCAB.find(dc_constants.GAP_OR_PAD)
+  pad_token = dc_constants.SEQ_VOCAB.find(dc_constants.GAP)
   y_pred_lens = tf.reduce_sum(tf.cast(y_pred != pad_token, y_pred.dtype), -1)
   return y_pred, y_pred_lens
 
@@ -660,13 +683,15 @@ class AlignmentMetric(tf.keras.metrics.Metric):
     gap_extend_penalty: The penalty of extending gaps (-e).
   """
 
-  def __init__(self,
-               matching_score: float = 2.0,
-               mismatch_penalty: float = 5.0,
-               gap_open_penalty: float = 5.0,
-               gap_extend_penalty: float = 4.0,
-               name: str = 'alignment_metric',
-               **kwargs):
+  def __init__(
+      self,
+      matching_score: float = 2.0,
+      mismatch_penalty: float = 5.0,
+      gap_open_penalty: float = 5.0,
+      gap_extend_penalty: float = 4.0,
+      name: str = 'alignment_metric',
+      **kwargs,
+  ):
     super(AlignmentMetric, self).__init__(name=name, **kwargs)
     self.matching_score = matching_score
     self.mismatch_penalty = mismatch_penalty
@@ -713,7 +738,8 @@ class AlignmentMetric(tf.keras.metrics.Metric):
     tf.debugging.assert_equal(
         x=b,
         y=tf.shape(y_pred)[0],
-        message='y_true and y_pred must have the same batch size.')
+        message='y_true and y_pred must have the same batch size.',
+    )
     m, n = tf.shape(y_true)[1], tf.shape(y_pred)[1]
     # Defines an appropriate large positive float to represent "infinity".
     inf = tf.convert_to_tensor(1e9, dtype)  # TODO: float16 support?
@@ -735,14 +761,16 @@ class AlignmentMetric(tf.keras.metrics.Metric):
         y_true,
         y_pred,
         matching_score=matching_score,
-        mismatch_penalty=mismatch_penalty)
+        mismatch_penalty=mismatch_penalty,
+    )
     wavefrontified_subs_costs = wavefrontify(subs_costs)
     # Stacks gap penalties as tf.Tensor of shape [3, 1, 1] for broadcasting.
     gap_pens = tf.stack([gap_open, gap_open, gap_extend])[:, None, None]
 
     # Setups reduction operators.
-    def reduce_max_with_argmax(t: tf.Tensor,
-                               axis: int = 0) -> Tuple[tf.Tensor, tf.Tensor]:
+    def reduce_max_with_argmax(
+        t: tf.Tensor, axis: int = 0
+    ) -> Tuple[tf.Tensor, tf.Tensor]:
       # Note(fllinares): I haven't yet managed to beat the performance of this
       # (wasteful) implementation with tf.argmax + tf.gather / tf.gather_nd :(
       t_max = tf.reduce_max(t, axis=axis)
@@ -771,8 +799,9 @@ class AlignmentMetric(tf.keras.metrics.Metric):
             tf.pad(
                 tf.fill([1, m - 1, b], -inf),
                 paddings=[[0, 0], [1, 0], [0, 0]],
-                constant_values=tf.convert_to_tensor(0.0, dtype=dtype)),
-            tf.fill([2, m, b], -inf)
+                constant_values=tf.convert_to_tensor(0.0, dtype=dtype),
+            ),
+            tf.fill([2, m, b], -inf),
         ],
         axis=0,
     )
@@ -786,14 +815,17 @@ class AlignmentMetric(tf.keras.metrics.Metric):
         tf.pad(
             tf.fill([m, b], -inf),
             paddings=[[1, 0], [0, 0]],
-            constant_values=-gap_open),
+            constant_values=-gap_open,
+        ),
         tf.roll(
             tf.pad(
                 tf.fill([m, b], -inf),
                 paddings=[[1, 0], [0, 0]],
-                constant_values=-gap_open),
+                constant_values=-gap_open,
+            ),
             shift=1,
-            axis=0),
+            axis=0,
+        ),
     ])
     # Allocates memory for backtracking "directions" tensor.
     dir_all = tf.TensorArray(tf.int32, size=m + n + 1, clear_after_read=True)
@@ -807,8 +839,9 @@ class AlignmentMetric(tf.keras.metrics.Metric):
             tf.pad(
                 tf.fill([1, m, b], -2),
                 paddings=[[0, 0], [1, 0], [0, 0]],
-                constant_values=tf.convert_to_tensor(-1, dtype=tf.int32)),
-            tf.fill([2, m + 1, b], -2)
+                constant_values=tf.convert_to_tensor(-1, dtype=tf.int32),
+            ),
+            tf.fill([2, m + 1, b], -2),
         ],
         axis=0,
     )
@@ -823,14 +856,17 @@ class AlignmentMetric(tf.keras.metrics.Metric):
         tf.pad(
             tf.fill([m, b], -2),
             paddings=[[1, 0], [0, 0]],
-            constant_values=tf.convert_to_tensor(0, dtype=tf.int32)),
+            constant_values=tf.convert_to_tensor(0, dtype=tf.int32),
+        ),
         tf.roll(
             tf.pad(
                 tf.fill([m, b], -2),
                 paddings=[[1, 0], [0, 0]],
-                constant_values=tf.convert_to_tensor(0, dtype=tf.int32)),
+                constant_values=tf.convert_to_tensor(0, dtype=tf.int32),
+            ),
             shift=1,
-            axis=0),
+            axis=0,
+        ),
     ])
     dir_all = dir_all.write(1, tf.cast(dir_all_p1, tf.int32))
 
@@ -920,8 +956,9 @@ class AlignmentMetric(tf.keras.metrics.Metric):
     #   insert extend (3): {insert} -> {insert}.
     #   delete open (4): {match, insert} -> {delete}.
     #   delete extend (5): {delete} -> {delete}.
-    trans_enc = tf.constant([[1, 1, 1], [2, 3, 2], [4, 4, 5]],
-                            dtype=tf.int32)  # [m_curr, m_prev]
+    trans_enc = tf.constant(
+        [[1, 1, 1], [2, 3, 2], [4, 4, 5]], dtype=tf.int32
+    )  # [m_curr, m_prev]
     # Initializes additional backtracking variables.
     k_opt = k_end  # [b], next antidiagonal for backtracking.
     i_opt = y_true_lens  # [b], next col for backtracking.
@@ -945,8 +982,9 @@ class AlignmentMetric(tf.keras.metrics.Metric):
       # Computes tentative next sparse updates for paths tensor, safeguarding
       # against invalid indexing.
       safe_m_opt_n = tf.maximum(m_opt_n, 0)
-      edges_n = tf.gather_nd(trans_enc,
-                             tf.stack([safe_m_opt, safe_m_opt_n], axis=-1))
+      edges_n = tf.gather_nd(
+          trans_enc, tf.stack([safe_m_opt, safe_m_opt_n], axis=-1)
+      )
       paths_sp_n = tf.stack([samp_idx, i_opt, k_opt - i_opt, edges_n], -1)
 
       # Checks if start (0, 0) was reached during backtracking.
@@ -957,8 +995,9 @@ class AlignmentMetric(tf.keras.metrics.Metric):
       k_opt = tf.where(cond, k_opt_n, k_opt)
       i_opt = tf.where(cond, i_opt_n, i_opt)
       m_opt = tf.where(cond, m_opt_n, m_opt)
-      paths_sp_k = tf.where(cond[:, None], paths_sp_n, tf.zeros([b, 4],
-                                                                tf.int32))
+      paths_sp_k = tf.where(
+          cond[:, None], paths_sp_n, tf.zeros([b, 4], tf.int32)
+      )
       paths_sp = paths_sp.write(k, paths_sp_k)  # [0, 0, 0, 0] used as dummy up.
 
     # Applies sparse updates, building paths tensor.
@@ -985,17 +1024,21 @@ class AlignmentMetric(tf.keras.metrics.Metric):
         'num_correct_matches': sum_positions(correct_matches),
     }
     metric_values['alignment_length'] = (
-        metric_values['num_matches'] + metric_values['num_insertions'] +
-        metric_values['num_deletions'])
+        metric_values['num_matches']
+        + metric_values['num_insertions']
+        + metric_values['num_deletions']
+    )
     # Computes percent identity (PID). PID is defined as 1.0 in the particular
     # case in which the ground-truth and predicted sequences are "empty", i.e.,
     # consist only of gap tokens.
     unsafe_pid = (
-        metric_values['num_correct_matches'] /
-        metric_values['alignment_length'])
-    metric_values['pid'] = tf.where(metric_values['alignment_length'] > 0,
-                                    tf.cast(unsafe_pid, dtype),
-                                    tf.convert_to_tensor(1.0, dtype))
+        metric_values['num_correct_matches'] / metric_values['alignment_length']
+    )
+    metric_values['pid'] = tf.where(
+        metric_values['alignment_length'] > 0,
+        tf.cast(unsafe_pid, dtype),
+        tf.convert_to_tensor(1.0, dtype),
+    )
 
     return v_opt, paths, metric_values
 
@@ -1015,6 +1058,115 @@ class AlignmentMetric(tf.keras.metrics.Metric):
     self._pid.reset_states()
 
 
+def get_batch_identity_ccs_pred(
+    ccs: tf.Tensor,
+    y_pred: tf.Tensor,
+    y_true: tf.Tensor,
+    alignment_metric: AlignmentMetric,
+) -> Tuple[tf.Tensor, tf.Tensor]:
+  """Calculate identity for CCS and sequence predicted by DeepConsensus.
+
+  Args:
+    ccs: A tf.Tensor of size [batch, max_length] representing CCS sequence.
+    y_pred: A tf.Tensor of size [batch, max_length, vocab_size] representing the
+      predicted tokens.
+    y_true: A tf.Tensor of size [batch, max_length] representing the true
+      sequence.
+    alignment_metric: AlignmentMetric object for computing alignment to the true
+      sequence for identity calculation.
+
+  Returns:
+    A tuple (identity_ccs, identity_pred) such that
+      + identity_ccs is a tf.Tensor of size [] that contains proportion
+          identity (between 0 and 1) for the CCS sequence over the batch.
+      + identity_pred is a tf.Tensor of size [] that contains proportion
+          identity (between 0 and 1) of predicted sequence over batch.
+  """
+  # Calculate identity for the predicted sequence.
+  _, _, metric_values_pred = alignment_metric.alignment(y_true, y_pred)
+  identity_pred = per_batch_identity(metric_values_pred)
+
+  # Calculate identity for CCS sequence.
+  ccs = tf.cast(ccs, tf.int32)
+  # Convert CCS to one-hot to match the shape of expected alignment inputs.
+  ccs_one_hot = tf.one_hot(
+      ccs, depth=len(dc_constants.SEQ_VOCAB), dtype=dc_constants.TF_DATA_TYPE
+  )
+
+  _, _, metric_values_ccs = alignment_metric.alignment(y_true, ccs_one_hot)
+  identity_ccs = per_batch_identity(metric_values_ccs)
+  return identity_ccs, identity_pred
+
+
+def per_batch_identity(metric_values: Mapping[str, tf.Tensor]) -> tf.Tensor:
+  """Calculates identity over the whole batch given metrics."""
+  tot_alignment_length = tf.reduce_sum(metric_values['alignment_length'])
+  unsafe_pid = (
+      tf.reduce_sum(metric_values['num_correct_matches']) / tot_alignment_length
+  )
+  # PID is defined as 1.0 in the particular case in which all the ground-truth
+  # and all predicted sequences are "empty", i.e., consist only of gap tokens.
+  if tf.equal(tot_alignment_length, 0):
+    return tf.convert_to_tensor(1.0, dtype=tf.float32)
+  return tf.cast(unsafe_pid, dtype=tf.float32)
+
+
+class YieldOverCCSMetric(tf.keras.metrics.Metric):
+  """Computes yield over ccs metric for a given quality threshold.
+
+  Attributes:
+    quality_threshold: a float value for thersholding DC and CCS identity.
+    yield_dc: DeepConsensus yield (number of batches that have identity >=
+      quality threshold).
+    yield_ccs: CCS yield.
+  """
+
+  def __init__(
+      self,
+      quality_threshold: float = 0.997,
+      name: str = 'yield_over_ccs',
+      **kwargs,
+  ):
+    super(YieldOverCCSMetric, self).__init__(name=name, **kwargs)
+    self.quality_threshold = quality_threshold
+    self.yield_dc = self.add_weight(
+        name='yield_dc', shape=[], initializer='zeros'
+    )
+    self.yield_ccs = self.add_weight(
+        name='yield_ccs', shape=[], initializer='zeros'
+    )
+
+  def update_state(
+      self, identity_ccs: tf.Tensor, identity_pred: tf.Tensor
+  ) -> None:
+    """Updates DeepConsensus and CCS yield based on provided identities.
+
+    Args:
+      identity_ccs: A tf.Tensor of size [] that contains proportion identity
+        (between 0 and 1) for the CCS sequence over the batch.
+      identity_pred: A tf.Tensor of size [] that contains proportion identity
+        (between 0 and 1) of predicted sequence over batch.
+    """
+    yield_dc = tf.cast(
+        identity_pred >= self.quality_threshold, dtype=tf.float32
+    )
+    yield_ccs = tf.cast(
+        identity_ccs >= self.quality_threshold, dtype=tf.float32
+    )
+    self.yield_dc.assign_add(yield_dc)
+    self.yield_ccs.assign_add(yield_ccs)
+
+  def result(self) -> tf.Tensor:
+    """Calculates yield of DeepConsensus over CCS using accumulated yields."""
+    # Avoid division by 0.
+    return tf.math.divide_no_nan(self.yield_dc, self.yield_ccs)
+
+  def reset_state(self) -> None:
+    """Resets DeepConsensus and CCS yield back to 0."""
+    self.yield_dc.assign(0.0)
+    self.yield_ccs.assign(0.0)
+
+
 class DistillationLoss(tf.keras.losses.Loss):
   """Computes the distillation loss between the student and teacher logits.
 
@@ -1033,12 +1185,16 @@ class DistillationLoss(tf.keras.losses.Loss):
   def __init__(
       self,
       temperature: float = 1.0,
-      reduction: tf.keras.losses.Reduction = tf.keras.losses.Reduction.AUTO):
+      logit_loss: tf.keras.losses.Loss = tf.keras.losses.get('kl_divergence'),
+      reduction: tf.keras.losses.Reduction = tf.keras.losses.Reduction.AUTO,
+  ):
     super(DistillationLoss, self).__init__(reduction=reduction)
     self.temperature = temperature
+    self.logit_loss = logit_loss
 
-  def call(self, teacher_logits: tf.Tensor,
-           student_logits: tf.Tensor) -> tf.Tensor:
+  def call(
+      self, teacher_logits: tf.Tensor, student_logits: tf.Tensor
+  ) -> tf.Tensor:
     """Computes the distillation loss between student and teacher logits.
 
     Args:
@@ -1052,6 +1208,6 @@ class DistillationLoss(tf.keras.losses.Loss):
     """
     teacher_probs = tf.nn.softmax(teacher_logits / self.temperature, axis=-1)
     student_probs = tf.nn.softmax(student_logits / self.temperature, axis=-1)
-    kl = tf.keras.losses.kld(teacher_probs, student_probs)
+    loss = self.logit_loss(teacher_probs, student_probs)
     # Take the mean across the positions in the sequence.
-    return tf.math.reduce_mean(kl, axis=-1)
+    return tf.math.reduce_mean(loss, axis=-1)
