@@ -40,13 +40,19 @@ from deepconsensus.utils import utils
 class DCModelOutput:
   molecule_name: str
   window_pos: int
+  ec: float
+  np_num_passes: int
+  rq: float
+  rg: str
   sequence: Optional[str] = None
   quality_string: Optional[str] = None
 
 
-def get_full_sequence(deepconsensus_outputs: Iterable[DCModelOutput],
-                      example_width: int,
-                      fill_n: bool = False):
+def get_full_sequence(
+    deepconsensus_outputs: Iterable[DCModelOutput],
+    max_length: int,
+    fill_n: bool = False,
+):
   """Stitch together windows of predictions into a full sequence."""
   # Build up the full sequence from the sorted windows.
   full_sequence_parts = []
@@ -60,36 +66,35 @@ def get_full_sequence(deepconsensus_outputs: Iterable[DCModelOutput],
         return None, ''
       else:
         # Add N-base filler for sequences that were unable to be inferred.
-        full_sequence_parts.append('N' * example_width)
-        empty_quality_scores = np.array([dc_constants.EMPTY_QUAL] *
-                                        example_width)
+        full_sequence_parts.append('N' * max_length)
+        empty_quality_scores = np.array([dc_constants.EMPTY_QUAL] * max_length)
         empty_quality_string = utils.quality_scores_to_string(
-            empty_quality_scores)
+            empty_quality_scores
+        )
         quality_string_parts.append(empty_quality_string)
-        start += example_width
+        start += max_length
     full_sequence_parts.append(dc_output.sequence)
     quality_string_parts.append(dc_output.quality_string)
-    start += example_width
+    start += max_length
   full_sequence = ''.join(full_sequence_parts)
   full_quality_string = ''.join(quality_string_parts)
   return full_sequence, full_quality_string
 
 
-def remove_gaps_and_padding(sequence: str,
-                            quality_string: str) -> Tuple[str, str]:
-  """Removes gaps/padding and corresponding quality score from outputs."""
-  # Remove padding and gaps from the final sequence.
+def remove_gaps(sequence: str, quality_string: str) -> Tuple[str, str]:
+  """Removes gaps and corresponding quality score from outputs."""
+  # Remove gaps from the final sequence.
   final_sequence = ''
   final_quality_string = ''
-  bases_to_remove = set([dc_constants.GAP_OR_PAD])
-  # Only keep bases and quality scores for non padding and non gap positions.
+  bases_to_remove = set([dc_constants.GAP])
+  # Only keep bases and quality scores for non gap positions.
   for base, quality in zip(sequence, quality_string):
     if base not in bases_to_remove:
       final_sequence += base
       final_quality_string += quality
 
   assert len(final_sequence) == len(final_quality_string)
-  assert dc_constants.GAP_OR_PAD not in final_sequence
+  assert dc_constants.GAP not in final_sequence
   return final_sequence, final_quality_string
 
 
@@ -104,8 +109,9 @@ def is_quality_above_threshold(quality_string, min_quality):
   return rounded_avg_phred >= min_quality
 
 
-def format_as_fastq(molecule_name: str, sequence: str,
-                    quality_string: str) -> str:
+def format_as_fastq(
+    molecule_name: str, sequence: str, quality_string: str
+) -> str:
   formatted_for_fastq = f'@{molecule_name}\n'
   formatted_for_fastq += f'{sequence}\n'
   formatted_for_fastq += '+\n'
@@ -116,55 +122,68 @@ def format_as_fastq(molecule_name: str, sequence: str,
 @dataclasses.dataclass
 class OutcomeCounter:
   empty_sequence: int = 0
-  only_gaps_and_padding: int = 0
+  only_gaps: int = 0
   failed_quality_filter: int = 0
   failed_length_filter: int = 0
   success: int = 0
 
 
-def stitch_to_fastq(molecule_name: str, predictions: Iterable[DCModelOutput],
-                    example_width: int, min_quality: int, min_length: int,
-                    outcome_counter: OutcomeCounter) -> Optional[str]:
+def stitch_to_fastq(
+    molecule_name: str,
+    predictions: Iterable[DCModelOutput],
+    max_length: int,
+    min_quality: int,
+    min_length: int,
+    outcome_counter: OutcomeCounter,
+) -> Optional[str]:
   """Stitch windows of predictions together, filter, and make FASTQ string."""
   full_sequence, full_quality_string = get_full_sequence(
-      deepconsensus_outputs=predictions, example_width=example_width)
+      deepconsensus_outputs=predictions, max_length=max_length
+  )
   # Filter out the read if it is empty after stitching.
   if not full_sequence:
     outcome_counter.empty_sequence += 1
-    logging.vlog(1, 'Filtered out read that was empty after stitching: %s',
-                 molecule_name)
+    logging.vlog(
+        1, 'Filtered out read that was empty after stitching: %s', molecule_name
+    )
 
     return None
 
-  final_sequence, final_quality_string = remove_gaps_and_padding(
-      sequence=full_sequence, quality_string=full_quality_string)
-  # Filter out the read if it contains only gaps or padding and no bases.
+  final_sequence, final_quality_string = remove_gaps(
+      sequence=full_sequence, quality_string=full_quality_string
+  )
+  # Filter out the read if it contains only gaps and no bases.
   if not final_sequence:
-    outcome_counter.only_gaps_and_padding += 1
+    outcome_counter.only_gaps += 1
     logging.vlog(
         1,
-        'Filtered out read that contained only gaps/padding and no bases: %s',
-        molecule_name)
+        'Filtered out read that contained only gaps and no bases: %s',
+        molecule_name,
+    )
     return None
 
   # Filter out the read if its quality scores are too low.
   if not is_quality_above_threshold(
-      quality_string=final_quality_string, min_quality=min_quality):
+      quality_string=final_quality_string, min_quality=min_quality
+  ):
     outcome_counter.failed_quality_filter += 1
-    logging.vlog(1, 'Filtered out read below quality threshold: %s',
-                 molecule_name)
+    logging.vlog(
+        1, 'Filtered out read below quality threshold: %s', molecule_name
+    )
     return None
 
   # Filter out the read if it is too short.
   if len(final_sequence) < min_length:
     outcome_counter.failed_length_filter += 1
-    logging.vlog(1, 'Filtered out read below length threshold: %s',
-                 molecule_name)
+    logging.vlog(
+        1, 'Filtered out read below length threshold: %s', molecule_name
+    )
     return None
 
   fastq = format_as_fastq(
       molecule_name=molecule_name,
       sequence=final_sequence,
-      quality_string=final_quality_string)
+      quality_string=final_quality_string,
+  )
   outcome_counter.success += 1
   return fastq
