@@ -43,17 +43,22 @@ def setUpModule():
   logging.set_verbosity(logging.FATAL)
 
 
-def fake_model_output(start: int, window_size: int, padding: int):
+def fake_model_output(start: int, window_size: int):
   return stitch_utils.DCModelOutput(
       molecule_name='name',
       window_pos=start,
-      sequence=''.join(random.choices('ACGT', k=window_size)) + '-' * padding,
-      quality_string='!' * (window_size + padding))
+      sequence=''.join(random.choices('ACGT', k=window_size)),
+      quality_string='!' * window_size,
+      ec=2.5,
+      np_num_passes=2,
+      rq=0.98,
+      rg='test_rg',
+  )
 
 
-def fake_model_outputs(window_size: int, num_windows: int, padding: int = 0):
+def fake_model_outputs(window_size: int, num_windows: int):
   outputs = [
-      fake_model_output(start=start, window_size=window_size, padding=padding)
+      fake_model_output(start=start, window_size=window_size)
       for start in range(0, window_size * num_windows, window_size)
   ]
   return outputs
@@ -66,12 +71,15 @@ class GetFullSequenceTest(parameterized.TestCase):
     width = 5
     dc_outputs = fake_model_outputs(window_size=width, num_windows=10)
     expected_sequence = ''.join(
-        [dc_output.sequence for dc_output in dc_outputs])
+        [dc_output.sequence for dc_output in dc_outputs]
+    )
     expected_quality_string = ''.join(
-        [dc_output.quality_string for dc_output in dc_outputs])
+        [dc_output.quality_string for dc_output in dc_outputs]
+    )
 
     sequence_output, quality_output = stitch_utils.get_full_sequence(
-        deepconsensus_outputs=dc_outputs, example_width=width)
+        deepconsensus_outputs=dc_outputs, max_length=width
+    )
     self.assertEqual(expected_sequence, sequence_output)
     self.assertEqual(expected_quality_string, quality_output)
 
@@ -86,57 +94,66 @@ class GetFullSequenceTest(parameterized.TestCase):
       if n == rand_seq_knockout:
         dc_output.sequence = 'N' * width
         dc_output.quality_string = utils.quality_score_to_string(
-            dc_constants.EMPTY_QUAL) * len(dc_output.sequence)
+            dc_constants.EMPTY_QUAL
+        ) * len(dc_output.sequence)
     expected_sequence = ''.join(
-        [dc_output.sequence for dc_output in dc_outputs])
+        [dc_output.sequence for dc_output in dc_outputs]
+    )
     expected_quality_string = ''.join(
-        [dc_output.quality_string for dc_output in dc_outputs])
+        [dc_output.quality_string for dc_output in dc_outputs]
+    )
     # Knockout sequence
     dc_outputs.pop(rand_seq_knockout)
 
     sequence_output, quality_output = stitch_utils.get_full_sequence(
-        deepconsensus_outputs=dc_outputs, example_width=width, fill_n=True)
+        deepconsensus_outputs=dc_outputs, max_length=width, fill_n=True
+    )
     self.assertEqual(expected_sequence, sequence_output)
     self.assertEqual(expected_quality_string, quality_output)
 
 
-class RemoveGapsAndPaddingTest(parameterized.TestCase):
+class RemoveGapsTest(parameterized.TestCase):
 
   @parameterized.named_parameters(
       dict(
-          testcase_name='no gaps/padding',
+          testcase_name='no gaps',
           sequence='ATCG',
           quality_string=utils.quality_scores_to_string(np.array([1, 2, 3, 4])),
           expected_sequence='ATCG',
           expected_quality_string=utils.quality_scores_to_string(
-              np.array([1, 2, 3, 4])),
+              np.array([1, 2, 3, 4])
+          ),
       ),
       dict(
-          testcase_name='some gaps/padding',
+          testcase_name='some gaps',
           sequence='AT CG ',
           quality_string=utils.quality_scores_to_string(
-              np.array([1, 2, 3, 4, 5, 6])),
+              np.array([1, 2, 3, 4, 5, 6])
+          ),
           expected_sequence='ATCG',
           expected_quality_string=utils.quality_scores_to_string(
-              np.array([1, 2, 4, 5])),
+              np.array([1, 2, 4, 5])
+          ),
       ),
       dict(
-          testcase_name='all gaps/padding',
+          testcase_name='all gaps',
           sequence='    ',
           quality_string=utils.quality_scores_to_string(np.array([1, 2, 3, 4])),
           expected_sequence=None,
           expected_quality_string=None,
       ),
   )
-  def test_remove_gaps_and_padding(self, sequence, quality_string,
-                                   expected_sequence, expected_quality_string):
+  def test_remove_gaps(
+      self, sequence, quality_string, expected_sequence, expected_quality_string
+  ):
     if expected_sequence:
       expected_output = (expected_sequence, expected_quality_string)
     else:
       expected_output = ('', '')
 
-    output = stitch_utils.remove_gaps_and_padding(
-        sequence=sequence, quality_string=quality_string)
+    output = stitch_utils.remove_gaps(
+        sequence=sequence, quality_string=quality_string
+    )
     self.assertEqual(output, expected_output)
 
 
@@ -147,20 +164,24 @@ class IsQualityAboveThresholdTest(parameterized.TestCase):
       dict(min_quality=20, read_qualities=(20, 20, 20, 20), should_pass=True),
       dict(min_quality=40, read_qualities=(40, 40, 40, 40), should_pass=True),
       # Average phred is not the same as average base quality.
-      dict(min_quality=40, read_qualities=(39, 39, 41, 41), should_pass=False)
+      dict(min_quality=40, read_qualities=(39, 39, 41, 41), should_pass=False),
   ])
-  def test_is_quality_above_threshold(self, min_quality, read_qualities,
-                                      should_pass):
+  def test_is_quality_above_threshold(
+      self, min_quality, read_qualities, should_pass
+  ):
     quality_string = [utils.quality_score_to_string(x) for x in read_qualities]
 
     passes = stitch_utils.is_quality_above_threshold(
-        quality_string=quality_string, min_quality=min_quality)
+        quality_string=quality_string, min_quality=min_quality
+    )
     self.assertEqual(passes, should_pass)
 
 
 class ConvertToFastqStrDoFnTest(absltest.TestCase):
 
-  def assert_correct_fastq_str(self, molecule_name, fasta_str):
+  def assert_correct_fastq_str(
+      self, molecule_name, fasta_str, max_base_quality
+  ):
     fasta_str_parts = fasta_str.split('\n')
     contig_name = fasta_str_parts[0]
     # Check that the contig name is formatted correctly.
@@ -168,14 +189,15 @@ class ConvertToFastqStrDoFnTest(absltest.TestCase):
     sequence_line = fasta_str_parts[1]
     separator_line = fasta_str_parts[2]
     quality_string_line = fasta_str_parts[3]
-    padding_and_gap = [dc_constants.GAP_OR_PAD, dc_constants.GAP_OR_PAD]
+    gap = [dc_constants.GAP, dc_constants.GAP]
     # Check the sequence line contains only valid bases.
-    self.assertNoCommonElements(set(sequence_line), padding_and_gap)
+    self.assertNoCommonElements(set(sequence_line), gap)
     self.assertEqual(separator_line, '+')
     # Not all values in this range are allowed, since we are binning, but we
     # do not consider that for this test.
     possible_quals = utils.quality_scores_to_string(
-        np.array(range(dc_constants.EMPTY_QUAL, dc_constants.MAX_QUAL + 1)))
+        np.array(range(dc_constants.EMPTY_QUAL, max_base_quality + 1))
+    )
     self.assertContainsSubset(quality_string_line, possible_quals)
     self.assertLen(quality_string_line, len(sequence_line))
 
@@ -188,9 +210,12 @@ class ConvertToFastqStrDoFnTest(absltest.TestCase):
     output = stitch_utils.format_as_fastq(
         molecule_name=molecule_name,
         sequence=sequence,
-        quality_string=quality_string)
+        quality_string=quality_string,
+    )
 
-    self.assert_correct_fastq_str(molecule_name=molecule_name, fasta_str=output)
+    self.assert_correct_fastq_str(
+        molecule_name=molecule_name, fasta_str=output, max_base_quality=93
+    )
 
 
 if __name__ == '__main__':
