@@ -36,7 +36,6 @@ Usage:
     --truth_split=truth_split.tsv \
     --output=examples-@split.tfrecord.gz \
     --cpus=4
-
 """
 
 import collections
@@ -46,13 +45,14 @@ import multiprocessing
 import multiprocessing.pool
 import os
 import time
-from typing import Dict, List
+from typing import Counter, Dict, List, Tuple, Union
 
 from absl import flags
 from absl import logging
+import numpy as np
 import tensorflow as tf
 
-from deepconsensus.preprocess import utils
+from deepconsensus.preprocess import pre_lib
 from deepconsensus.utils import dc_constants
 from absl import app
 
@@ -64,29 +64,67 @@ Issue = dc_constants.Issue
 
 FLAGS = flags.FLAGS
 
-flags.DEFINE_string('subreads_to_ccs', None,
-                    'Input BAM containing subreads aligned to ccs.')
+flags.DEFINE_string(
+    'subreads_to_ccs', None, 'Input BAM containing subreads aligned to ccs.'
+)
 flags.DEFINE_string('ccs_fasta', None, 'Input FASTA containing ccs sequences.')
 flags.DEFINE_string('ccs_bam', None, 'Input BAM containing ccs sequences.')
 flags.DEFINE_string(
-    'output', None,
-    ('Output filename. If training, use @split wildcard for split name. '
-     'For example: --output=examples-@split.tfrecord.gz'
-     'The output filename must end in .tfrecord.gz'))
+    'output',
+    None,
+    (
+        'Output filename. If training, use @split wildcard for split name. '
+        'For example: --output=examples-@split.tfrecord.gz'
+        'The output filename must end in .tfrecord.gz'
+    ),
+)
 flags.DEFINE_string('truth_to_ccs', None, 'Input truth bam aligned to ccs.')
 flags.DEFINE_string('truth_bed', None, 'Input truth bedfile.')
-# TODO TODO
-flags.DEFINE_string('truth_split', None,
-                    'Input file defining train/eval/test splits.')
+# TODO
+flags.DEFINE_string(
+    'truth_split', None, 'Input file defining train/eval/test splits.'
+)
 flags.DEFINE_integer(
     'cpus',
     multiprocessing.cpu_count(),
-    'Number of worker processes to use. Use 0 to disable parallel processing. '
-    'Minimum of 2 CPUs required for parallel processing.',
-    short_name='j')
-flags.DEFINE_integer('bam_reader_threads', 8,
-                     'Number of decompression threads to use.')
+    (
+        'Number of worker processes to use. Use 0 to disable parallel'
+        ' processing. Minimum of 2 CPUs required for parallel processing.'
+    ),
+    short_name='j',
+)
+flags.DEFINE_integer(
+    'bam_reader_threads', 8, 'Number of decompression threads to use.'
+)
 flags.DEFINE_integer('limit', 0, 'Limit processing to n ZMWs.')
+flags.DEFINE_integer(
+    'ins_trim',
+    5,
+    (
+        'Trim insertions greater than ins_trim bp in subreads to 0bp.'
+        'No trimming if flag is set to 0'
+    ),
+)
+flags.DEFINE_bool(
+    'use_ccs_smart_windows',
+    False,
+    (
+        'If true, CCS smart window widths are used to partition '
+        'subreads into windows.'
+    ),
+)
+_USE_CCS_BQ = flags.DEFINE_bool(
+    'use_ccs_bq',
+    False,
+    'If true, incorporate CCS Base Quality scores into tf.examples.',
+)
+# The following just need to match the training parameters.
+_MAX_PASSES = flags.DEFINE_integer(
+    'max_passes', 20, 'Maximum subreads in each input.'
+)
+_EXAMPLE_WIDTH = flags.DEFINE_integer(
+    'max_length', 100, 'Number of bases in each input.'
+)
 
 
 def register_required_flags():
@@ -103,11 +141,11 @@ def trace_exception(f):
   @functools.wraps(f)
   def wrap(*args, **kwargs):
     try:
-      result = f(*args, **kwargs)
+      result = f(*args, **kwargs)  # pytype: disable=wrong-arg-types  # always-use-return-annotations
       return result
-    except:  # pylint: disable=bare-except
+    except Exception as exc:
       logging.exception('Error in function %s.', f.__name__)
-      raise Exception('Error in worker process')
+      raise exc
 
   return wrap
 
@@ -117,8 +155,9 @@ def make_dirs(path):
   tf.io.gfile.makedirs(os.path.dirname(path))
 
 
-def setup_writers(output_fname: str,
-                  splits: List[str]) -> Dict[str, tf.io.TFRecordWriter]:
+def setup_writers(
+    output_fname: str, splits: List[str]
+) -> Dict[str, tf.io.TFRecordWriter]:
   """Creates tf writers for split set."""
   tf_writers = {}
   tf_ops = tf.io.TFRecordOptions(compression_type='GZIP')
@@ -130,8 +169,11 @@ def setup_writers(output_fname: str,
   return tf_writers
 
 
-def write_tf_record(tf_example_str_set: List[bytes], split: str,
-                    tf_writers: Dict[str, tf.io.TFRecordWriter]):
+def write_tf_record(
+    tf_example_str_set: List[bytes],
+    split: str,
+    tf_writers: Dict[str, tf.io.TFRecordWriter],
+):
   """Writes tf examples to a split."""
   for tf_example_str in tf_example_str_set:
     tf_writers[split].write(tf_example_str)
@@ -139,8 +181,9 @@ def write_tf_record(tf_example_str_set: List[bytes], split: str,
 
 
 @trace_exception
-def tf_record_writer(output_fname: str, splits: List[str],
-                     queue: Queue) -> bool:
+def tf_record_writer(
+    output_fname: str, splits: List[str], queue: Queue
+) -> bool:
   """tf_record writing worker."""
   tf_writers = setup_writers(output_fname, splits)
   while True:
@@ -154,15 +197,20 @@ def tf_record_writer(output_fname: str, splits: List[str],
 
 
 @trace_exception
-def process_subreads(subreads: List[utils.Read],
-                     ccs_seqname: str,
-                     dc_config: utils.DcConfig,
-                     split: str,
-                     queue: Queue,
-                     local=False):
+def process_subreads(
+    subreads: List[pre_lib.Read],
+    ccs_seqname: str,
+    dc_config: pre_lib.DcConfig,
+    split: str,
+    window_widths: np.ndarray,
+    queue: Queue,
+    local: bool = False,
+) -> Union[Counter[str], Tuple[List[str], str, Counter[str]]]:
   """Subread processing worker."""
   tf_out = []
-  dc_example = utils.subreads_to_dc_example(subreads, ccs_seqname, dc_config)
+  dc_example = pre_lib.subreads_to_dc_example(
+      subreads, ccs_seqname, dc_config, window_widths
+  )
   for example in dc_example.iter_examples():
     tf_out.append(example.tf_example().SerializeToString())
   dc_example.counter[f'n_examples_{split}'] += len(tf_out)
@@ -175,8 +223,9 @@ def process_subreads(subreads: List[utils.Read],
   return dc_example.counter
 
 
-def clear_tasks(tasks: List[AsyncResult],
-                main_counter: collections.Counter) -> List[AsyncResult]:
+def clear_tasks(
+    tasks: List[AsyncResult], main_counter: collections.Counter
+) -> List[AsyncResult]:
   """Clear successful tasks and log result."""
   for task in tasks:
     if task.ready():
@@ -196,8 +245,10 @@ def main(unused_argv) -> None:
     raise ValueError('Must set cpus to 0 or >=2 for parallel processing.')
 
   if FLAGS.ccs_fasta:
-    raise NotImplementedError('The --ccs_fasta flag has been deprecated. '
-                              'Please use --ccs_bam instead.')
+    raise NotImplementedError(
+        'The --ccs_fasta flag has been deprecated. '
+        'Please use --ccs_bam instead.'
+    )
 
   is_training = FLAGS.truth_to_ccs and FLAGS.truth_bed and FLAGS.truth_split
 
@@ -206,14 +257,16 @@ def main(unused_argv) -> None:
 
   if is_training:
     logging.info('Generating tf.Examples in training mode.')
-    contig_split = utils.read_truth_split(FLAGS.truth_split)
+    contig_split = pre_lib.read_truth_split(FLAGS.truth_split)
     splits = set(contig_split.values())
     for split in splits:
       if '@split' not in FLAGS.output:
         raise ValueError('You must add @split to --output when training.')
   elif FLAGS.truth_to_ccs or FLAGS.truth_bed or FLAGS.truth_split:
-    err_msg = ('You must specify truth_to_ccs, truth_bed, and truth_split '
-               'to generate a training dataset.')
+    err_msg = (
+        'You must specify truth_to_ccs, truth_bed, and truth_split '
+        'to generate a training dataset.'
+    )
     raise Exception(err_msg)
   else:
     logging.info('Generating tf.Examples in inference mode.')
@@ -222,24 +275,32 @@ def main(unused_argv) -> None:
   manager = multiprocessing.Manager()
   queue = manager.Queue()
 
-  dc_config = utils.DcConfig(max_passes=20, example_width=100, padding=20)
+  dc_config = pre_lib.DcConfig(
+      max_passes=_MAX_PASSES.value,
+      max_length=_EXAMPLE_WIDTH.value,
+      use_ccs_bq=_USE_CCS_BQ.value,
+  )
 
-  proc_feeder, main_counter = utils.create_proc_feeder(
+  proc_feeder, main_counter = pre_lib.create_proc_feeder(
       subreads_to_ccs=FLAGS.subreads_to_ccs,
       ccs_bam=FLAGS.ccs_bam,
       dc_config=dc_config,
+      ins_trim=FLAGS.ins_trim,
+      use_ccs_smart_windows=FLAGS.use_ccs_smart_windows,
       truth_bed=FLAGS.truth_bed,
       truth_to_ccs=FLAGS.truth_to_ccs,
       truth_split=FLAGS.truth_split,
       limit=FLAGS.limit,
-      bam_reader_threads=FLAGS.bam_reader_threads)
+      bam_reader_threads=FLAGS.bam_reader_threads,
+  )
 
   if FLAGS.cpus == 0:
     logging.info('Using a single cpu.')
     tf_writers = setup_writers(FLAGS.output, splits)
     for args in proc_feeder():
       tf_example_str_set, split, counter = process_subreads(
-          *args, queue=None, local=True)
+          *args, queue=None, local=True
+      )
       write_tf_record(tf_example_str_set, split, tf_writers)
       # Update counter
       main_counter.update(counter)
@@ -249,8 +310,9 @@ def main(unused_argv) -> None:
     logging.info('Processing in parallel using %s cores', FLAGS.cpus)
     with multiprocessing.Pool(FLAGS.cpus) as pool:
       # Setup parallelization
-      tf_writer = pool.apply_async(tf_record_writer,
-                                   (FLAGS.output, splits, queue))
+      tf_writer = pool.apply_async(
+          tf_record_writer, (FLAGS.output, splits, queue)
+      )
       tasks = []
       for args in proc_feeder():
         tasks.append(pool.starmap_async(process_subreads, ([*args, queue],)))
@@ -271,24 +333,34 @@ def main(unused_argv) -> None:
   # Write summary
   logging.info('Completed processing %s ZMWs.', main_counter['n_zmw_pass'])
   summary_name = 'training' if is_training else 'inference'
-  dataset_summary = FLAGS.output.replace('.tfrecord.gz',
-                                         f'.{summary_name}.json')
+  dataset_summary = FLAGS.output.replace(
+      '.tfrecord.gz', f'.{summary_name}.json'
+  )
   # Remove @split from filenames
   dataset_summary = dataset_summary.replace('@split', 'summary')
   logging.info('Writing %s.', dataset_summary)
   make_dirs(dataset_summary)
-  with open(dataset_summary, 'w') as summary_file:
+  with tf.io.gfile.GFile(dataset_summary, 'w') as summary_file:
     summary = dict(main_counter.items())
     summary.update(dc_config.to_dict())
     flag_list = [
-        'subreads_to_ccs', 'ccs_bam', 'truth_to_ccs', 'truth_bed', 'truth_split'
+        'subreads_to_ccs',
+        'ccs_bam',
+        'truth_to_ccs',
+        'truth_bed',
+        'truth_split',
+        'max_passes',
+        'max_length',
+        'ins_trim',
     ]
     for flag in flag_list:
-      summary[flag] = FLAGS[flag].value
+      # Encode these as strings to ensure aggregation does not add values.
+      summary[flag] = str(FLAGS[flag].value)
     summary['version'] = dc_constants.__version__
     json_summary = json.dumps(summary, indent=True)
     summary_file.write(json_summary)
 
+
 if __name__ == '__main__':
   logging.use_python_logging()
   app.run(main)
