@@ -32,11 +32,12 @@ import os
 
 from absl import flags
 from absl.testing import absltest
+from absl.testing import flagsaver
 from absl.testing import parameterized
-import tensorflow as tf
 
+from deepconsensus.preprocess import pre_lib
 from deepconsensus.preprocess import preprocess
-from deepconsensus.preprocess import utils
+from deepconsensus.utils import test_utils
 from deepconsensus.utils.test_utils import deepconsensus_testdata
 from absl import app
 
@@ -51,18 +52,10 @@ def load_summary(tmp_dir, path):
   return json.load(open(summary_path, 'r'))
 
 
-def load_dataset(output, dataset):
-  # Load inference, train, eval, or test tfrecord.gz files.
-  tf_record = output.replace('@split', dataset)
-  dataset = tf.data.TFRecordDataset(tf_record, compression_type='GZIP')
-  examples = list(dataset.as_numpy_iterator())
-  return examples
-
-
 def get_unique_zmws(examples):
   zmws = []
   for example in examples:
-    features = utils.tf_example_to_features_dict(example)
+    features = pre_lib.tf_example_to_features_dict(example)
     zmws.append(int(features['name'].split('/')[1]))
   return len(set(zmws))
 
@@ -81,14 +74,14 @@ class PreprocessE2E(parameterized.TestCase):
     output = os.path.join(tmp_dir, 'tf-@split.tfrecord.gz')
     FLAGS.output = output
     preprocess.main([])
-    examples = load_dataset(output, 'inference')
-    features = utils.tf_example_to_features_dict(examples[0], inference=True)
+    examples = test_utils.load_dataset(output, 'inference')
+    features = pre_lib.tf_example_to_features_dict(examples[0], inference=True)
 
     # Check that window_pos incr. monotonically for each ZMW.
     last_pos = -1
     last_zmw = -1
     for example in examples:
-      features = utils.tf_example_to_features_dict(example, inference=True)
+      features = pre_lib.tf_example_to_features_dict(example, inference=True)
       zmw = int(features['name'].split('/')[1])
       if zmw != last_zmw:
         last_zmw = zmw
@@ -118,16 +111,16 @@ class PreprocessE2E(parameterized.TestCase):
     output = os.path.join(tmp_dir, 'tf-@split.tfrecord.gz')
     FLAGS.output = output
     preprocess.main([])
-    train_examples = load_dataset(output, 'train')
-    eval_examples = load_dataset(output, 'eval')
-    test_examples = load_dataset(output, 'test')
+    train_examples = test_utils.load_dataset(output, 'train')
+    eval_examples = test_utils.load_dataset(output, 'eval')
+    test_examples = test_utils.load_dataset(output, 'test')
     all_examples = train_examples + eval_examples + test_examples
 
     # Check that window_pos incr. monotonically for each ZMW.
     last_pos = -1
     last_zmw = -1
     for example in all_examples:
-      features = utils.tf_example_to_features_dict(example, inference=False)
+      features = pre_lib.tf_example_to_features_dict(example, inference=False)
       zmw = int(features['name'].split('/')[1])
       if zmw != last_zmw:
         last_zmw = zmw
@@ -147,8 +140,9 @@ class PreprocessE2E(parameterized.TestCase):
     n_zmw_eval = get_unique_zmws(eval_examples)
     n_zmw_test = get_unique_zmws(test_examples)
     self.assertLessEqual(summary['n_zmw_pass'], n_zmws)
-    self.assertEqual(n_zmw_train + n_zmw_eval + n_zmw_test,
-                     summary['n_zmw_pass'])
+    self.assertEqual(
+        n_zmw_train + n_zmw_eval + n_zmw_test, summary['n_zmw_pass']
+    )
     self.assertEqual(n_zmw_train, summary['n_zmw_train'])
     self.assertEqual(n_zmw_eval, summary['n_zmw_eval'])
     self.assertEqual(n_zmw_test, summary['n_zmw_test'])
@@ -158,11 +152,52 @@ class PreprocessE2E(parameterized.TestCase):
     self.assertLen(eval_examples, summary['n_examples_eval'])
     self.assertLen(test_examples, summary['n_examples_test'])
 
-    features = utils.tf_example_to_features_dict(train_examples[0])
+    features = pre_lib.tf_example_to_features_dict(train_examples[0])
     self.assertIn('label', features)
     self.assertIn('label/shape', features)
-    self.assertSameElements(features['subreads'].shape,
-                            features['subreads/shape'])
+    self.assertSameElements(
+        features['subreads'].shape, features['subreads/shape']
+    )
+
+  def test_invalid_tf_examples(self):
+    """Tests for proper error thrown when loading improprer tf example."""
+    output = os.path.join(self.create_tempdir(), 'tf-@split.tfrecord.gz')
+    with flagsaver.flagsaver(
+        subreads_to_ccs=testdata('human_1m/subreads_to_ccs.bam'),
+        ccs_bam=testdata('human_1m/ccs.bam'),
+        use_ccs_bq=True,
+        cpus=0,
+        limit=1,
+        output=output,
+    ):
+      preprocess.main([])
+      examples = test_utils.load_dataset(output, 'train')
+      with self.assertRaisesRegex(ValueError, 'Invalid subreads shape'):
+        _ = pre_lib.tf_example_to_features_dict(
+            examples[0], inference=True, use_ccs_bq=False
+        )
+
+  def test_bq_tf_examples(self):
+    """Tests preprocessing inference with base quality score features."""
+    output = os.path.join(self.create_tempdir(), 'tf-@split.tfrecord.gz')
+    with flagsaver.flagsaver(
+        subreads_to_ccs=testdata('human_1m/subreads_to_ccs.bam'),
+        ccs_bam=testdata('human_1m/ccs.bam'),
+        use_ccs_bq=True,
+        cpus=0,
+        limit=1,
+        output=output,
+    ):
+      preprocess.main([])
+      examples = test_utils.load_dataset(output, 'inference')
+
+      features = pre_lib.tf_example_to_features_dict(
+          examples[0],
+          inference=True,
+          use_ccs_bq=True,
+      )
+      self.assertEqual(list(features['subreads/shape']), [86, 100, 1])
+      self.assertEqual(list(features['subreads'].shape), [86, 100, 1])
 
 
 if __name__ == '__main__':
