@@ -36,7 +36,6 @@ import tensorflow as tf
 from deepconsensus.models import data_providers
 from deepconsensus.utils import dc_constants
 
-
 WRITE_NORMAL = '\x1b[0m'
 WRITE_GREEN_BACKGROUND = '\x1b[102m'
 WRITE_RED_BACKGROUND = '\x1b[101m'
@@ -46,13 +45,14 @@ KMER_SIZE = 10
 
 
 def remove_gaps(seq: str) -> str:
-  """Removes gaps and padding from sequences."""
-  seq = seq.replace(dc_constants.GAP_OR_PAD, '')
+  """Removes gaps from sequences."""
+  seq = seq.replace(dc_constants.GAP, '')
   return seq
 
 
 def get_deepconsensus_prediction(
-    model: tf.keras.Model, rows: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
+    model: tf.keras.Model, rows: tf.Tensor
+) -> Tuple[tf.Tensor, tf.Tensor]:
   """Runs model on given rows and returns distributions and predictions."""
   softmax_output = model(rows, training=False)
   pred = tf.argmax(softmax_output, axis=-1)
@@ -66,17 +66,23 @@ def check_has_errors(label: str, pred: str) -> bool:
 
 def ints_to_bases(bases_row: tf.Tensor) -> str:
   """Converts ints to bases based on order in the vocab."""
-  return ''.join([dc_constants.VOCAB[int(b)] for b in bases_row])
+  return ''.join([dc_constants.SEQ_VOCAB[int(b)] for b in bases_row])
 
 
-def convert_to_bases(rows: tf.Tensor, label: tf.Tensor,
-                     deepconsensus_pred: tf.Tensor,
-                     max_passes: int) -> Tuple[List[str], str, str]:
+def convert_to_bases(
+    rows: tf.Tensor,
+    label: tf.Tensor,
+    deepconsensus_pred: tf.Tensor,
+    max_passes: int,
+) -> Tuple[List[str], str, str]:
   """Converts numerical tensors to string of bases."""
   rows = tf.squeeze(rows)
   label = tf.squeeze(label)
   deepconsensus_pred = tf.squeeze(deepconsensus_pred)
-  base_indices, _, _, _, _, _ = data_providers.get_indices(max_passes)
+  base_indices, _, _, _, _, _, _ = data_providers.get_indices(
+      max_passes,
+      use_ccs_bq=False,
+  )
   subread_rows_range = range(*base_indices)
   subread_rows = [rows[i, :].numpy() for i in subread_rows_range]
   subread_rows = [row for row in subread_rows if np.sum(row) != 0]
@@ -115,9 +121,9 @@ def pretty_print_proto(dc_input, print_aux=False):
       print('%4d' % read.subread_strand * len(read.bases))
 
 
-def get_results_df(experiments: List[int],
-                   experiment_pattern: str,
-                   decimals: int = 5) -> pd.DataFrame:
+def get_results_df(
+    experiments: List[int], experiment_pattern: str, decimals: int = 5
+) -> pd.DataFrame:
   """Returns a dataframe with inference results."""
   all_lines = None
   for experiment in experiments:
@@ -145,7 +151,9 @@ def get_results_df(experiments: List[int],
 def get_results_df_compact(df: pd.DataFrame) -> pd.DataFrame:
   """Returns a compact version of the results with fewer columns."""
   cols_to_keep = [
-      'dataset_type', 'experiment_and_work_unit', 'accuracy',
-      'per_example_accuracy'
+      'dataset_type',
+      'experiment_and_work_unit',
+      'accuracy',
+      'per_example_accuracy',
   ]
   return df[cols_to_keep]
