@@ -33,18 +33,20 @@ import pysam
 import tensorflow as tf
 
 # DeepConsensus Version
-__version__ = '0.3.1'
+__version__ = '1.2.0'
 
 # Vocab
-GAP_OR_PAD = ' '
+GAP = ' '
 ALLOWED_BASES = 'ATCG'
-VOCAB = GAP_OR_PAD + ALLOWED_BASES
+SEQ_VOCAB = GAP + ALLOWED_BASES
+SEQ_VOCAB_SIZE = len(SEQ_VOCAB)
 
 # Value used to fill in empty rows in the tf.Examples.
-GAP_OR_PAD_INT = VOCAB.index(GAP_OR_PAD)
+GAP_INT = SEQ_VOCAB.index(GAP)
 
 PYSAM_READ_ADVANCING_OPS = list(
-    map(int, [pysam.CMATCH, pysam.CINS, pysam.CEQUAL, pysam.CDIFF]))
+    map(int, [pysam.CMATCH, pysam.CINS, pysam.CEQUAL, pysam.CDIFF])
+)
 
 
 class Issue(int, enum.Enum):
@@ -68,11 +70,13 @@ CIGAR_OPS = {
     'P': pysam.CPAD,
     '=': pysam.CEQUAL,
     'X': pysam.CDIFF,
-    'B': pysam.CBACK
+    'B': pysam.CBACK,
 }
 
 # Defining this as ints makes comparison operations faster.
+PYSAM_CMATCH = int(pysam.CMATCH)
 PYSAM_CINS = int(pysam.CINS)
+PYSAM_CDEL = int(pysam.CDEL)
 PYSAM_CSOFT_CLIP = int(pysam.CSOFT_CLIP)
 PYSAM_CHARD_CLIP = int(pysam.CHARD_CLIP)
 
@@ -80,32 +84,47 @@ PYSAM_CHARD_CLIP = int(pysam.CHARD_CLIP)
 TF_DATA_TYPE = tf.float32
 NP_DATA_TYPE = np.float32
 
-PW_MAX = 9
-IP_MAX = 9
-SN_MAX = 15
-STRAND_MAX = 2
-
 # E. Coli eval region is first 10% of the genome,
 # Test region is last 10% of the genome
 # Total genome length is 4642522.
 ECOLI_REGIONS = {
     'TRAIN': (464253, 4178270),
     'EVAL': (0, 464252),
-    'TEST': (4178271, 4642522)
+    'TEST': (4178271, 4642522),
+}
+TRAIN_REGIONS = {
+    'HUMAN': (
+        [str(i) for i in range(1, 19)]
+        + ['chr%d' % i for i in range(1, 19)]
+        + ['X', 'Y', 'chrX', 'chrY']
+    ),
+    'MAIZE': [str(i) for i in range(1, 9)] + ['chr%d' % i for i in range(1, 9)],
+}
+
+EVAL_REGIONS = {
+    'HUMAN': ['21', '22', 'chr21', 'chr22'],
+    'MAIZE': ['9', 'chr9'],
+}
+TEST_REGIONS = {
+    'HUMAN': ['19', '20', 'chr19', 'chr20'],
+    'MAIZE': ['10', 'chr10'],
 }
-# chrs 1-18, X, and Y. All with and without 'chr'.
-HUMAN_TRAIN_REGIONS = [str(i) for i in range(1, 19)] + [
-    'chr%d' % i for i in range(1, 19)
-] + ['X', 'Y', 'chrX', 'chrY']
-# chrs 21 and 22, both with and without 'chr'.
-HUMAN_EVAL_REGIONS = ['21', '22', 'chr21', 'chr22']
-HUMAN_TEST_REGIONS = ['19', '20', 'chr19', 'chr20']
 
 # List of features in DC examples.
 DC_FEATURES = [
-    'rows', 'label', 'num_passes', 'window_pos', 'name',
-    'ccs_base_quality_scores'
+    'rows',
+    'label',
+    'num_passes',
+    'window_pos',
+    'name',
+    'ccs_base_quality_scores',
+    'ec',
+    'np_num_passes',
+    'rq',
+    'rg',
 ]
 
-MAX_QUAL = 40
 EMPTY_QUAL = 0
+
+# The name of the eval metric to optimize (choosing checkpoints and vizier).
+MAIN_EVAL_METRIC_NAME = 'eval/per_example_accuracy'
