@@ -28,11 +28,12 @@
 """Utilities to help with testing code."""
 
 import os
-from typing import List, Text, Tuple, Union
+from typing import Dict, List, Text, Tuple, Union
 
 from absl import flags
 from absl.testing import absltest
 import numpy as np
+import tensorflow as tf
 
 from deepconsensus.utils import dc_constants
 from tensorflow.python.platform import gfile
@@ -107,16 +108,19 @@ def deepconsensus_testdata(filename):
     The absolute path to a testdata file.
   """
   return genomics_testdata(
-      os.path.join('deepconsensus/testdata', filename), DEEPCONSENSUS_DATADIR)
+      os.path.join('deepconsensus/testdata', filename), DEEPCONSENSUS_DATADIR
+  )
 
 
 def get_one_hot(value: Union[int, np.ndarray]) -> np.ndarray:
   """Returns a one-hot vector for a given value."""
-  return np.eye(len(dc_constants.VOCAB), dtype=dc_constants.NP_DATA_TYPE)[value]
+  return np.eye(len(dc_constants.SEQ_VOCAB), dtype=dc_constants.NP_DATA_TYPE)[
+      value
+  ]
 
 
 def seq_to_array(seq: str) -> List[int]:
-  return [dc_constants.VOCAB.index(i) for i in seq]
+  return [dc_constants.SEQ_VOCAB.index(i) for i in seq]
 
 
 def multiseq_to_array(sequences: Union[Text, List[Text]]) -> np.ndarray:
@@ -142,9 +146,16 @@ def convert_seqs(sequences: List[str]) -> Tuple[np.ndarray, np.ndarray]:
   Returns:
     y_true as array
     y_pred_scores as probability array
-
   """
   y_true, y_pred_scores = sequences
   y_true = multiseq_to_array(y_true).astype(dc_constants.NP_DATA_TYPE)
   y_pred_scores = seq_to_one_hot(y_pred_scores)
   return y_true, y_pred_scores
+
+
+def load_dataset(output: str, dataset: str) -> List[Dict[str, np.ndarray]]:
+  # Load inference, train, eval, or test tfrecord.gz files.
+  tf_record = output.replace('@split', dataset)
+  dataset = tf.data.TFRecordDataset(tf_record, compression_type='GZIP')
+  examples = list(dataset.as_numpy_iterator())
+  return examples
